import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Objects;

import javax.swing.*;
import javax.swing.text.Segment;

/**
 * Client-server graphical editor
 *
 * @author Chris Bailey-Kellogg, Dartmouth CS 10, Fall 2012; loosely based on CS 5 code by Tom Cormen
 * @author CBK, winter 2014, overall structure substantially revised
 * @author Travis Peters, Dartmouth CS 10, Winter 2015; remove EditorCommunicatorStandalone (use echo server for testing)
 * @author CBK, spring 2016 and Fall 2016, restructured Shape and some of the GUI
 * @author tucker simpson, logan tucker
 */

public class Editor extends JFrame {
	private static String serverIP = "localhost";			// IP address of sketch server
	// "localhost" for your own machine;
	// or ask a friend for their IP address

	private static final int width = 800, height = 800;		// canvas size

	// Current settings on GUI
	public enum Mode {
		DRAW, MOVE, RECOLOR, DELETE
	}
	private Mode mode = Mode.DRAW;				// drawing/moving/recoloring/deleting objects
	private String shapeType = "ellipse";		// type of object to add
	private Color color = Color.black;			// current drawing color

	// Drawing state
	// these are remnants of my implementation; take them as possible suggestions or ignore them
	private Shape curr = null;					// current shape (if any) being drawn
	private Sketch sketch;						// holds and handles all the completed objects
	private int movingId = -1;					// current shape id (if any; else -1) being moved
	private Point drawTo = null;				// where the drawing started
	private Point originalFrom = null;			// where object was when move/draw started
	private Point moveFrom = null;
	private Point moveTo = null;				// where object is as it's being dragged


	// Communication
	private EditorCommunicator comm;			// communication with the sketch server

	public Editor() {
		super("Graphical Editor");

		sketch = new Sketch();

		// Connect to server
		comm = new EditorCommunicator(serverIP, this);
		comm.start();

		// Helpers to create the canvas and GUI (buttons, etc.)
		JComponent canvas = setupCanvas();
		JComponent gui = setupGUI();

		// Put the buttons and canvas together into the window
		Container cp = getContentPane();
		cp.setLayout(new BorderLayout());
		cp.add(canvas, BorderLayout.CENTER);
		cp.add(gui, BorderLayout.NORTH);

		// Usual initialization
		setLocationRelativeTo(null);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		pack();
		setVisible(true);
	}

	/**
	 * Creates a component to draw into
	 */
	private JComponent setupCanvas() {
		JComponent canvas = new JComponent() {
			public void paintComponent(Graphics g) {
				super.paintComponent(g);
				drawSketch(g);
			}
		};

		canvas.setPreferredSize(new Dimension(width, height));

		canvas.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent event) {
				handlePress(event.getPoint());
			}

			public void mouseReleased(MouseEvent event) {
				handleRelease();
			}
		});

		canvas.addMouseMotionListener(new MouseAdapter() {
			public void mouseDragged(MouseEvent event) {
				handleDrag(event.getPoint());
			}
		});

		return canvas;
	}

	/**
	 * Creates a panel with all the buttons
	 */
	private JComponent setupGUI() {
		// Select type of shape
		String[] shapes = {"ellipse", "freehand", "rectangle", "segment"};
		JComboBox<String> shapeB = new JComboBox<String>(shapes);
		shapeB.addActionListener(e -> shapeType = (String)((JComboBox<String>)e.getSource()).getSelectedItem());

		// Select drawing/recoloring color
		// Following Oracle example
		JButton chooseColorB = new JButton("choose color");
		JColorChooser colorChooser = new JColorChooser();
		JLabel colorL = new JLabel();
		colorL.setBackground(Color.black);
		colorL.setOpaque(true);
		colorL.setBorder(BorderFactory.createLineBorder(Color.black));
		colorL.setPreferredSize(new Dimension(25, 25));
		JDialog colorDialog = JColorChooser.createDialog(chooseColorB,
				"Pick a Color",
				true,  //modal
				colorChooser,
				e -> { color = colorChooser.getColor(); colorL.setBackground(color); },  // OK button
				null); // no CANCEL button handler
		chooseColorB.addActionListener(e -> colorDialog.setVisible(true));

		// Mode: draw, move, recolor, or delete
		JRadioButton drawB = new JRadioButton("draw");
		drawB.addActionListener(e -> mode = Mode.DRAW);
		drawB.setSelected(true);
		JRadioButton moveB = new JRadioButton("move");
		moveB.addActionListener(e -> mode = Mode.MOVE);
		JRadioButton recolorB = new JRadioButton("recolor");
		recolorB.addActionListener(e -> mode = Mode.RECOLOR);
		JRadioButton deleteB = new JRadioButton("delete");
		deleteB.addActionListener(e -> mode = Mode.DELETE);
		ButtonGroup modes = new ButtonGroup(); // make them act as radios -- only one selected
		modes.add(drawB);
		modes.add(moveB);
		modes.add(recolorB);
		modes.add(deleteB);
		JPanel modesP = new JPanel(new GridLayout(1, 0)); // group them on the GUI
		modesP.add(drawB);
		modesP.add(moveB);
		modesP.add(recolorB);
		modesP.add(deleteB);

		// Put all the stuff into a panel
		JComponent gui = new JPanel();
		gui.setLayout(new FlowLayout());
		gui.add(shapeB);
		gui.add(chooseColorB);
		gui.add(colorL);
		gui.add(modesP);
		return gui;
	}

	/**
	 * Getter for the sketch instance variable
	 */
	public Sketch getSketch() {
		return sketch;
	}


	/**
	 * Draws all the shapes in the sketch,
	 * along with the object currently being drawn in this editor (not yet part of the sketch)
	 */
	public void drawSketch(Graphics g) {
		// TODO: YOUR CODE HERE

		// Having concurrent modification bug.
		// for each shape in the sketch map of shapes.NavigableKeyset() returns  oldest to newest (low to high keys)
		for (Integer id : sketch.getShapes().navigableKeySet()) {

			if (sketch.getShapes().get(id) != null) sketch.getShapes().get(id).draw(g);
		}
		if (curr != null) curr.draw(g);
	}

	// Helpers for event handlers

	/**
	 * Helper method for press at point
	 * In drawing mode, start a new object;
	 * in moving mode, (request to) start dragging if clicked in a shape;
	 * in recoloring mode, (request to) change clicked shape's color
	 * in deleting mode, (request to) delete clicked shape
	 */
	private void handlePress(Point p) {
		// TODO: YOUR CODE HERE
		System.out.println(sketch.getShapes().keySet());

		if(mode == Mode.DRAW){
			originalFrom = p;
			drawTo = p;

			// if shape type is valid
			if (Objects.equals(shapeType, "ellipse") || Objects.equals(shapeType, "freehand") ||
					Objects.equals(shapeType, "rectangle") || Objects.equals(shapeType, "segment")) {

				// create a new shape of shapeType
//				if (Objects.equals(shapeType, "ellipse")){
//					curr = new Ellipse(p.x, p.y, color);
//				}
				if (Objects.equals(shapeType, "freehand")){
					curr = new Polyline(p.x, p.y, color);
				}
//				if (Objects.equals(shapeType, "rectangle")){
//					curr = new Rectangle(p.x, p.y, color);
//				}
				if (Objects.equals(shapeType, "segment")){
					curr = new MySegment(p.x, p.y, color);
				}

				// send editor communicator new shape info
			}

			// remember current mouse press as start location for drags and moves
		}

		// if mode equals move
		else if (mode == Mode.MOVE){
			// find shape that was clicked and remember its ID and its location when it was clicked
			for(Integer id : sketch.getShapes().keySet()){
				if(sketch.getShapes().get(id).contains(p.x, p.y)){
					moveFrom = p;
					movingId = id;
				}
			}
		}

		// if mode equals recolor
		else if(mode == Mode.RECOLOR) {

			// find shape that was clicked on
			for (Integer id : sketch.getShapes().keySet()) {
				if (sketch.getShapes().get(id).contains(p.x, p.y)) {

					// tell editor communicator what shape to recolor and what color to color it
					comm.send("recolor " + id + " " + color.getRGB());
				}
			}
		}

		// if mode equals delete
		else if(mode == Mode.DELETE){

			// find shape that was clicked
			for (Integer id : sketch.getShapes().keySet()) {
				if (sketch.getShapes().get(id).contains(p.x, p.y)) {

					// tell communicator what shape to delete
					comm.send("delete " + id);
				}
			}
		}
	}

	/**
	 * Helper method for drag to new point
	 * In drawing mode, update the other corner of the object;
	 * in moving mode, (request to) drag the object
	 */
	private void handleDrag(Point p) {
		// TODO: YOUR CODE HERE

		// If mode = move and a shape was successfully clicked in during move earlier
		if (mode == Mode.MOVE && moveFrom != null) {

			// Find dx and dy
			int dx = p.x - moveFrom.x;
			int dy = p.y - moveFrom.y;

			// Request to drag the object
			comm.send("move "+movingId+" "+dx+" "+dy);
			moveFrom = p;
			repaint();

		}

		// IF in draw mode
		else if (mode == Mode.DRAW) {

			// Modify the shape locally rather than through requests
			if(shapeType.equals("ellipse")){
				curr = new Ellipse(originalFrom.x, originalFrom.y, color);
				curr.setCorners(originalFrom.x, originalFrom.y, p.x, p.y);
				drawTo = p;
				repaint();
			}
			else if(shapeType.equals("rectangle")){
				curr = new Rectangle(originalFrom.x, originalFrom.y, color);
				curr.setCorners(originalFrom.x, originalFrom.y, p.x, p.y);
				drawTo = p;
				repaint();
			}
			else if(shapeType.equals("segment")|shapeType.equals("freehand")) {
				curr.setEnd(p.x, p.y);
				repaint();
			}

		}

	}

	/**
	 * Helper method for release
	 * In drawing mode, pass the add new object request on to the server;
	 * in moving mode, release it
	 */
	private void handleRelease() {
		// If in draw mode and successfully drew a shape, send request to server to draw shape
		if (curr!= null && mode == Mode.DRAW) {
//			String message = curr.toString();
//			//System.out.println("string sent");
//			System.out.println(message);
			comm.send(curr.toString());
			curr = null;

		}

		// If in moving mode, forget which shape was being moved and where it was being moved from
		else if (mode == Mode.MOVE) {
			moveFrom = null;
			movingId = -1;
		}
		repaint();
	}

	// METHODS TO HANDLE REQUESTS FROM EDITOR_COMMUNICATOR

	// Handle request to add shape
	public void addShape (Shape incoming){
		sketch.addShape(incoming);
		repaint();
	}

	// Handle request to recolor
	public void recolor(int id, Color col){
		sketch.recolorShape(id, col);
		repaint();
	}

	// Handle request to delete shape
	public void delete(int id){
		sketch.delShape(id);
		repaint();
	}

	// Handle request to move shape
	public void move(int id, int dx, int dy){
		sketch.moveShape(id, dx, dy);
	}

	// Update new editors on the ID of the last item in the server sketch
	public void setCurrID(int currID){
		//System.out.println("Setting CurrID to: "+currID);
		sketch.setCurrId(currID);
		System.out.println("Curr ID is now: "+currID);
	}

	/**
	 * Take a String Array of generic parameters for rectangle/ellipse/segment and handle the parsing by parsing
	 * parameters and adding the correct shape to the sketch
	 */
	public void parseParams(String[] params) {
		int x1 = Integer.parseInt(params[3]);
		int y1 = Integer.parseInt(params[4]);
		int x2 = Integer.parseInt(params[5]);
		int y2 = Integer.parseInt(params[6]);
		int rgb = Integer.parseInt(params[7]);
		int id = Integer.parseInt(params[1]);
		if (params[2].equals("rectangle")) {
			Rectangle currRect = new Rectangle(x1, y1, x2, y2, new Color(rgb));
			getSketch().addShape(currRect, id);
		} else if (params[2].equals("ellipse")) {
			Ellipse currEl = new Ellipse(x1, y1, x2, y2, new Color(rgb));
			getSketch().addShape(currEl, id);
		} else {
			MySegment currSeg = new MySegment(x1, y1, x2, y2, new Color(rgb));
			getSketch().addShape(currSeg, id);
		}
	}

	/**
	 * Parse parameters of a freehand drawing and draw in local sketch
	 * @param line whole command, not yet split at any point
	 */
	private void parseFreehand(String line){
		// Get the color
		String[] typeAndColor = line.split(" ");
		Color col = new Color(Integer.parseInt(typeAndColor[1]));

		// Split into parameters of each segment
		String[] params = line.split("/");
		ArrayList<MySegment> segs = new ArrayList<>();

		for (int i = 1; i < params.length-1; i ++) {
			String[] coords = params[i].split(" ");
			int x1 = Integer.parseInt(coords[0]);
			int y1 = Integer.parseInt(coords[1]);
			int x2 = Integer.parseInt(coords[2]);
			int y2 = Integer.parseInt(coords[3]);
			segs.add(new MySegment(x1, y1, x2, y2, col));
		}

		getSketch().addShape(new Polyline(segs, col));
	}

	/**
	 * Handle the process of getting new editor up to speed on changes made to server sketch
	 * @param line shape ID and shape's to_string
	 */
	public void getCurrWorld(String line){
		String[] comm = line.split(" ");

		//System.out.println(line);
		if (comm[2].equals("rectangle") | comm[2].equals("ellipse") | comm[2].equals("segment")) {
			parseParams(comm);
		}

		else if (comm[2].equals("freehand")) {
			parseFreehand(line);
		}

		repaint();
		//System.out.println("CurrID: " + sketch.getCurrId());

	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new Editor();
			}
		});
	}
}
