import javax.swing.text.Segment;
import java.awt.*;
import java.io.*;
import java.net.Socket;
import java.util.ArrayList;

/**
 * Handles communication to/from the server for the editor
 * 
 * @author Chris Bailey-Kellogg, Dartmouth CS 10, Fall 2012
 * @author Chris Bailey-Kellogg; overall structure substantially revised Winter 2014
 * @author Travis Peters, Dartmouth CS 10, Winter 2015; remove EditorCommunicatorStandalone (use echo server for testing)
 * @author tucker simpson, logan tucker
 */
public class EditorCommunicator extends Thread {
	private PrintWriter out;		// to server
	private BufferedReader in;		// from server
	protected Editor editor;		// handling communication for

	/**
	 * Establishes connection and in/out pair
	 */
	public EditorCommunicator(String serverIP, Editor editor) {
		this.editor = editor;
		System.out.println("connecting to " + serverIP + "...");
		try {
			Socket sock = new Socket(serverIP, 4242);
			out = new PrintWriter(sock.getOutputStream(), true);
			in = new BufferedReader(new InputStreamReader(sock.getInputStream()));
			System.out.println("...connected");
		}
		catch (IOException e) {
			System.err.println("couldn't connect");
			System.exit(-1);
		}
	}

	/**
	 * Sends message to the server
	 */
	public void send(String msg) {
		out.println(msg);
	}

	/**
	 * Keeps listening for and handling (your code) messages from the server
	 */
	public void run() {
		try {
			String line;

			// Read line from server
			while ((line = in.readLine()) != null) {

				String[] comm = line.split(" ");

				// Get new editors up to speed if they join after edits have been made
				if (comm[0].equals("CurrID")) {
					editor.setCurrID(Integer.parseInt(comm[1]));
				}
				else if (comm[0].equals("sketch")) {
					editor.getCurrWorld(line);
				}

				// Handle command to add ellipse, rectangle or segment
				// Format: ["Shape_Type" x1 y1 x2 y2 rgb]
				if (comm[0].equals("ellipse") | comm[0].equals("rectangle") | comm[0].equals("segment")) {
					editor.addShape(parseParams(comm));
				}

				// Handle command to draw freehand
				else if (comm[0].equals("freehand")) {
					String[] segs = line.split("/");
					editor.addShape(parseFreehand(segs));
				}

				// Handle command to recolor
				else if (comm[0].equals("recolor")) {
					handleRecolor(comm);
				}

				// Handle command to delete
				else if (comm[0].equals("delete")) {
					handeDelete(comm);
				}

				else if (comm[0].equals("move")) {
					handleMove(comm);
				}

			}

		} catch(IOException e){
				e.printStackTrace();
		} finally{
				System.out.println("server hung up");
		}
	}

	/**
	 * Parse parameters and return shape for rectangle, ellipse, and segment
	 * @param params to_string of shape, separated at " "
	 * @return instance of shape
	 */
	private Shape parseParams(String[] params){
		int x1 = Integer.parseInt(params[1]);
		int y1 = Integer.parseInt(params[2]);
		int x2 = Integer.parseInt(params[3]);
		int y2 = Integer.parseInt(params[4]);
		int rgb = Integer.parseInt(params[5]);
		Color col = new Color(rgb);
		if (params[0].equals("ellipse")) {
			return new Ellipse(x1, y1, x2, y2, col);
		} else if(params[0].equals("segment")){
			return new MySegment(x1, y1, x2, y2, col);
		} else {
			return new Rectangle(x1, y1, x2, y2, col);
		}
	}

	/**
	 * Parse parameters for and return instance of freehand drawing
	 * @param params to_string of polyline, separated at every "/"
	 * @return freehand drawing
	 */
	private Polyline parseFreehand(String[] params){
		String typeAndColor = params[0];
		String[] splitForColor = typeAndColor.split(" ");
		Color col = new Color(Integer.parseInt(splitForColor[1]));

		ArrayList<MySegment> segs = new ArrayList<>();

		for (int i = 1; i < params.length; i ++) {
			String[] coords = params[i].split(" ");
			int x1 = Integer.parseInt(coords[0]);
			int y1 = Integer.parseInt(coords[1]);
			int x2 = Integer.parseInt(coords[2]);
			int y2 = Integer.parseInt(coords[3]);
			segs.add(new MySegment(x1, y1, x2, y2, col));
		}

		return new Polyline(segs, col);
	}

	/**
	 * forwards request to handle delete
	 * @param comm String[] containing command "delete" and ID of object to delete
	 */
	private void handeDelete(String[] comm){
		int id = Integer.parseInt(comm[1]);
		editor.delete(id);
	}

	/**
	 * forwards request to handle recolor
	 * @param comm String[] containing command "recolor" and ID of object to recolor
	 */
	private void handleRecolor(String[] comm){
		int id = Integer.parseInt(comm[1]);
		int rgb = Integer.parseInt(comm[2]);
		Color col = new Color(rgb);
		editor.recolor(id, col);
	}

	/**
	 * forwards request to handle move
	 * @param comm String[] containing command "move", ID of object to move, and how much to move it
	 */
	private void handleMove(String[] comm){
		int id = Integer.parseInt(comm[1]);
		int dx = Integer.parseInt(comm[2]);
		int dy = Integer.parseInt(comm[3]);
		editor.move(id, dx, dy);
	}


}
