import javax.swing.text.Segment;
import java.awt.*;
import java.io.*;
import java.net.Socket;
import java.sql.SQLOutput;
import java.util.ArrayList;
import java.util.Objects;
import java.util.TreeMap;

/**
 * Handles communication between the server and one client, for SketchServer
 *
 * @author Chris Bailey-Kellogg, Dartmouth CS 10, Fall 2012; revised Winter 2014 to separate SketchServerCommunicator
 * @author tucker simpson, logan tucker
 */
public class SketchServerCommunicator extends Thread {
	private Socket sock;					// to talk with client
	private BufferedReader in;				// from client
	private PrintWriter out;				// to client
	private SketchServer server;			// handling communication for

	public SketchServerCommunicator(Socket sock, SketchServer server) {
		this.sock = sock;
		this.server = server;
	}

	/**
	 * Sends a message to the client
	 * @param msg
	 */
	public void send(String msg) {
		out.println(msg);
	}
	
	/**
	 * Keeps listening for and handling (your code) messages from the client
	 */
	public void run() {
		try {
			System.out.println("someone connected");
			
			// Communication channel
			in = new BufferedReader(new InputStreamReader(sock.getInputStream()));
			out = new PrintWriter(sock.getOutputStream(), true);

			TreeMap<Integer, Shape> currState = server.getSketch().getShapes();
			if(!currState.isEmpty()) {
				server.broadcast("CurrID "+server.getSketch().getCurrId());
				for (Integer id : currState.descendingKeySet()) {
					server.broadcast("sketch " + id + " " + currState.get(id).toString());
				}
			}


			// go through all editor requests
			String l;
			while((l = in.readLine()) != null) {

				// parse information from request
				String[] req = l.split(" ");

				if (Objects.equals(req[0], "ellipse")) {

					// create new shape from request information
					// (creates shape but does not draw it, req[3] and req[4] are null)
					Ellipse newShape = new Ellipse(Integer.parseInt(req[1]), Integer.parseInt(req[2]),
							Integer.parseInt(req[3]), Integer.parseInt(req[4]), new Color(Integer.parseInt(req[5])));

					// tell server to add new shape to global sketch
					server.getSketch().addShape(newShape);
				}
				else if (req[0].equals("freehand")) {
					String[] segs = l.split("/");

					Polyline freehand = parseFreehand(segs);

					server.getSketch().addShape(freehand);
				}
				else if (Objects.equals(req[0], "rectangle")) {

					Rectangle newShape = new Rectangle(Integer.parseInt(req[1]), Integer.parseInt(req[2]),
							Integer.parseInt(req[3]), Integer.parseInt(req[4]), new Color(Integer.parseInt(req[5])));

					server.getSketch().addShape(newShape);
				}
				else if (Objects.equals(req[0], "segment")) {

					MySegment newShape = new MySegment(Integer.parseInt(req[1]), Integer.parseInt(req[2]),
							Integer.parseInt(req[3]), Integer.parseInt(req[4]), new Color(Integer.parseInt(req[5])));

					server.getSketch().addShape(newShape);
				}
				else if (Objects.equals(req[0], "move")) {
					handleMove(req);
					//server.getSketch().shapes.get(Integer.parseInt(req[1])).moveBy(Integer.parseInt(req[2]), Integer.parseInt(req[3]));
				}
				else if (Objects.equals(req[0], "delete")) {
					handeDelete(req);
					//server.getSketch().shapes.remove(Integer.parseInt(req[1]));
				}
				else if (Objects.equals(req[0], "recolor")) {
					handleRecolor(req);
					//System.out.println("item's ID: "+ req[1]);
					//System.out.println(server.getSketch().shapes.get(req[1]).toString());
					//server.getSketch().shapes.get(Integer.parseInt(req[1])).setColor(new Color(Integer.parseInt(req[req.length-1])));
				}

				// update sketch on global scale (broadcast  new sketch to all sketch server communicators
				server.broadcast(l);
			}

			// Clean up -- note that also remove self from server's list so it doesn't broadcast here
			server.removeCommunicator(this);
			out.close();
			in.close();
			sock.close();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Parse parameters for and return instance of freehand drawing
	 * @param params to_string of polyline, separated at every "/"
	 * @return freehand drawing
	 */
	private Polyline parseFreehand(String[] params){

		String typeAndColor = params[0];
		String[] splitForColor = typeAndColor.split(" ");
		Color col = new Color(Integer.parseInt(splitForColor[1]));

		ArrayList<MySegment> segs = new ArrayList<>();

		for (int i = 1; i < params.length; i ++) {
			String[] coords = params[i].split(" ");

			int x1 = Integer.parseInt(coords[0]);
			int y1 = Integer.parseInt(coords[1]);
			int x2 = Integer.parseInt(coords[2]);
			int y2 = Integer.parseInt(coords[3]);
			segs.add(new MySegment(x1, y1, x2, y2, col));
		}


		return new Polyline(segs, col);
	}

	/**
	 * forwards request to handle move
	 * @param comm String[] containing command "move", ID of object to move, and how much to move it
	 */
	public void handleMove(String[] comm){
		int id = Integer.parseInt(comm[1]);
		int dx = Integer.parseInt(comm[2]);
		int dy = Integer.parseInt(comm[3]);
		server.getSketch().moveShape(id, dx, dy);
	}

	/**
	 * forwards request to handle delete
	 * @param comm String[] containing command "delete" and ID of object to delete
	 */
	private void handeDelete(String[] comm){
		int id = Integer.parseInt(comm[1]);
		server.getSketch().delShape(id);
	}

	/**
	 * forwards request to handle recolor
	 * @param comm String[] containing command "recolor" and ID of object to recolor
	 */
	private void handleRecolor(String[] comm){
		int id = Integer.parseInt(comm[1]);
		int rgb = Integer.parseInt(comm[2]);
		Color col = new Color(rgb);
		server.getSketch().recolorShape(id, col);
	}

}
